#ifndef DeltaTouchInput_cpp
#define DeltaTouchInput_cpp

#include <Arduino.h>
#include <functional>
#include "RollingAverage.cpp"

class DeltaTouchInput {
	const unsigned int _debounceDelay = 100;

	int _pin;
	float _touchThreshold;
	std::function<void()> _callback;
	unsigned long _touchStart;
	bool _currentTouchValue, _prevTouchValue;
	RollingAverage _readings = RollingAverage(1000);

#ifdef TOUCH_DEBUG
	int _lastReading = -1;
	void printReading() {
		int reading = touchRead(_pin);
		if (reading != _lastReading) {
			Serial.print(reading);
			Serial.print("\t");
			Serial.print(_readings.getAverage());
			Serial.print("\t");
			Serial.print(reading / _readings.getAverage());
			Serial.print("\t");
			Serial.println(reading / _readings.getAverage() <= _touchThreshold);
		}
		_lastReading = reading;
	}
#endif

  public:
	DeltaTouchInput(int pin, float touchThreshold, std::function<void()> callback){
		_pin = pin;
		_touchThreshold = touchThreshold;
		_callback = callback;
	}

	void loop() {
		int reading = touchRead(_pin);
		bool touching = reading / _readings.getAverage() <= _touchThreshold;

		if (_prevTouchValue != touching) {
			_touchStart = millis();
		}

		if (millis() - _touchStart >= _debounceDelay) {
			_readings.add(reading);
			if (touching != _currentTouchValue) {
				_currentTouchValue = touching;
				if (touching)
					_callback();
			}
		}

		_prevTouchValue = touching;

#ifdef TOUCH_DEBUG
		printReading();
#endif
	}
};

#endif
