#ifndef EASING_cpp
#define EASING_cpp

enum EaseMode {
	LINEAR,
	QUAD_IN,
	QUAD_OUT,
	QUAD_IN_OUT,
	CUBIC_IN,
	CUBIC_OUT,
	CUBIC_IN_OUT,
	QUART_IN,
	QUART_OUT,
	QUART_IN_OUT,
	QUINT_IN,
	QUINT_OUT,
	QUINT_IN_OUT
};

class Easing
{
public:
	static float linear(float t) {
		return t;
	}

	static float easeInQuad(float t) {
		return t * t;
	}

	static float easeOutQuad(float t) {
		return t * (2 - t);
	}

	static float easeInOutQuad(float t) {
		return t < .5 ? 2 * t * t : -1 + (4 - 2 * t) * t;
	}

	static float easeInCubic(float t) {
		return t * t * t;
	}

	static float easeOutCubic(float t) {
		return (--t) * t * t + 1;
	}

	static float easeInOutCubic(float t) {
		return t < .5 ? 4 * t * t * t : (t - 1) * (2 * t - 2) * (2 * t - 2) + 1;
	}

	static float easeInQuart(float t) {
		return t * t * t * t;
	}

	static float easeOutQuart(float t) {
		return 1 - (--t) * t * t * t;
	}

	static float easeInOutQuart(float t) {
		return t < .5 ? 8 * t * t * t * t : 1 - 8 * (--t) * t * t * t;
	}

	static float easeInQuint(float t) {
		return t * t * t * t * t;
	}

	static float easeOutQuint(float t) {
		return 1 + (--t) * t * t * t * t;
	}

	static float easeInOutQuint(float t) {
		return t < .5 ? 16 * t * t * t * t * t : 1 + 16 * (--t) * t * t * t * t;
	}

	static float ease(EaseMode mode, float t) {
		switch (mode) {
			case EaseMode::QUAD_IN:
				return easeInQuad(t);
			case EaseMode::QUAD_OUT:
				return easeOutQuad(t);
			case EaseMode::QUAD_IN_OUT:
				return easeInOutQuad(t);
			case EaseMode::CUBIC_IN:
				return easeInCubic(t);
			case EaseMode::CUBIC_OUT:
				return easeOutCubic(t);
			case EaseMode::CUBIC_IN_OUT:
				return easeInOutCubic(t);
			case EaseMode::QUART_IN:
				return easeInQuart(t);
			case EaseMode::QUART_OUT:
				return easeOutQuart(t);
			case EaseMode::QUART_IN_OUT:
				return easeInOutQuart(t);
			case EaseMode::QUINT_IN:
				return easeInQuint(t);
			case EaseMode::QUINT_OUT:
				return easeOutQuint(t);
			case EaseMode::QUINT_IN_OUT:
				return easeInOutQuint(t);
			default:
				return linear(t);
		}
	}
};

#endif
