#ifndef LED_cpp
#define LED_cpp

#include <Arduino.h>
#include "LEDOutput.cpp"
#include "Easing.cpp"

class LED {
	std::string _name;
	LEDOutput* _output;
	bool _on;
	unsigned long _fadeDurationOn, _fadeDurationOff, _fadeStart;
	float _brightness, _targetBrightness, _startBrightness, _currentBrightness;
	EaseMode _easeMode;

	unsigned long getFadeDuration(){
		return _on ? _fadeDurationOn : _fadeDurationOff;
	}

	float getOutputMultiplier(){
		unsigned long duration = getFadeDuration();
		float elapsed = (millis() - _fadeStart) / (float)duration;

		if (elapsed >= 1)
			return _targetBrightness;

		_currentBrightness = (_targetBrightness - _startBrightness) * Easing::ease(_easeMode, elapsed) + _startBrightness;
		return _currentBrightness;
	}

	void reset(){
		_fadeStart = millis();
		_startBrightness = _currentBrightness;
		_targetBrightness = _on ? _brightness : 0;
	}

public:
	LED(std::string name, LEDOutput* output, unsigned long fadeDurationOn, unsigned long fadeDurationOff, EaseMode easeMode = EaseMode::LINEAR) {
		_name = name;
		_output = output;
		_fadeDurationOn = fadeDurationOn;
		_fadeDurationOff = fadeDurationOff;
		_brightness = 1.0f;
		_easeMode = easeMode;
		reset();
	}

	std::string getName(){
		return _name;
	}

	void on(){
		_on = true;
		reset();
	}

	void off(){
		_on = false;
		reset();
	}

	void toggle(){
		_on = !_on;
		reset();
	}

	void loop() {
		_output->writeFraction(getOutputMultiplier());
	}

	void setBrightness(float brightness){
		_brightness = brightness;
		reset();
	}
};

#endif
