#ifndef TouchInput_cpp
#define TouchInput_cpp

#include <Arduino.h>
#include <functional>

class TouchInput {
	const unsigned int _debounceDelay = 100;

	int _pin, _touchThreshold;
	std::function<void()> _callback;
	unsigned long _touchStart;
	bool _currentTouchValue, _prevTouchValue;

#ifdef TOUCH_DEBUG
	int _lastReading = -1;
	void printReading() {
		int reading = touchRead(_pin);
		if (reading != _lastReading)
			Serial.println(reading);
		_lastReading = reading;
	}
#endif

  public:
	TouchInput(int pin, int touchThreshold, std::function<void()> callback){
		_pin = pin;
		_touchThreshold = touchThreshold;
		_callback = callback;
	}

	void loop(){
		bool touching = touchRead(_pin) <= _touchThreshold;

		if (_prevTouchValue != touching) {
			_touchStart = millis();
		}

		if (millis() - _touchStart >= _debounceDelay && touching != _currentTouchValue){
			_currentTouchValue = touching;
			if (touching)
				_callback();
		}

		_prevTouchValue = touching;

#ifdef TOUCH_DEBUG
		printReading();
#endif
	}
};

#endif
