#include <Arduino.h>
#include "BinaryOutput.cpp"
#include "BinaryInput.cpp"
#include "PWMOutput.cpp"
#include "EspNowLightControlClient.cpp"
#include "LightControlPayload.cpp"
#include "Timer.cpp"

#define MOTION_DURATION 60000 // 1 minute
#define FAN_DURATION 300000 // 5 minutes
#define FAN_ON_DELAY 5000
#define BROADCAST_INTERVAL 500
#define BROADCAST_COUNT 3

EspNowLightControlClient _lightControl;
void sendPayload();
void resetBroadcast();
bool _motionActive = false;
BinaryInput _motionSensor(4);
PWMOutput _fan(23, 25000, 1);

int _broadcastCount = 0;

Timer _broadcastTimer([]() {
	if (++_broadcastCount >= BROADCAST_COUNT)
		return;
	sendPayload();
}, true);

Timer _fanStopTimer([]() {
	Serial.println("Fan stop");
	_fan.off();
});

Timer _motionStopTimer([]() {
	Serial.println("Motion stop");
	_motionActive = false;
	sendPayload();
	resetBroadcast();
});

void sendPayload() {
	LightControlPayload data = LightControlPayload();
	data.id = 3;
	data.brightness = 1;
	data.on = _motionActive;
	data.timer = 0;
	_lightControl.send(data);
	Serial.println(_motionActive);
}

void resetBroadcast() {
	_broadcastCount = 0;
	_broadcastTimer.reset(BROADCAST_INTERVAL);
}

void setup() {
	Serial.begin(115200);

	_lightControl.init();
	_broadcastTimer.reset(BROADCAST_INTERVAL);
	_fan.setValueFraction(0.5f);
}

void loop() {	
	if (_motionSensor.isHigh()) {
		if (_motionSensor.isHigh() != _motionActive) {
			Serial.println("Motion detected");
			_motionActive = true;
			sendPayload();
			resetBroadcast();
		}
		
		if (_motionSensor.millisSinceLastStateChange() >= FAN_ON_DELAY && _motionActive) {
			_fan.on();
		}
		
		_motionStopTimer.reset(MOTION_DURATION);
		_fanStopTimer.reset(FAN_DURATION);
	}

	_broadcastTimer.loop();
	_motionStopTimer.loop();
	_fanStopTimer.loop();
	_motionSensor.loop();
}
